[MESSAGES CONTROL]
# Disable the message(s) with the given id(s).
# http://pylint-messages.wikidot.com/all-codes
#
# C0103: Invalid name "%s" (should match %s)
# C0111: Missing docstring
# E1101: %s %r has no %r member
# R0901: Too many ancestors (%s/%s)
# R0902: Too many instance attributes (%s/%s)
# R0903: Too few public methods (%s/%s)
# R0904: Too many public methods (%s/%s)
# R0913: Too many arguments (%s/%s)
# R0915: Too many statements (%s/%s)
# W0141: Used builtin function %r
# W0142: Used * or ** magic
# W0221: Arguments number differs from %s method
# W0232: Class has no __init__ method
# W0401: Wildcard import %s
# W0613: Unused argument %r
# W0631: Using possibly undefined loop variable %r
#
disable = C0103,C0111,E1101,R0901,R0902,R0903,R0904,R0913,R0915,W0141,W0142,W0221,W0232,W0401,W0613,W0631

[TYPECHECK]
generated-members = REQUEST,acl_users,aq_parent,objects,DoesNotExist,_meta,status_code,content,context
